/******************************************************************************

Copyright (C) 2013-2017, Fujitsu Electronics Europe GmbH or a               
subsidiary of Fujitsu Electronics Europe GmbH.  All rights reserved.        
                                                                            
This software, including source code, documentation and related             
materials ("Software"), is owned by Fujitsu Electronics Europe GmbH or      
one of its subsidiaries ("Fujitsu").
                                                                            
If no EULA applies, Fujitsu hereby grants you a personal, non-exclusive,    
non-transferable license to copy, modify, and compile the                   
Software source code solely for use in connection with Fujitsu's            
integrated circuit products.  Any reproduction, modification, translation,  
compilation, or representation of this Software except as specified         
above is prohibited without the express written permission of Fujitsu.      
                                                                            
Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO                        
WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING,                        
BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED                                
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A                             
PARTICULAR PURPOSE. Fujitsu reserves the right to make                      
changes to the Software without notice. Fujitsu does not assume any         
liability arising out of the application or use of the Software or any      
product or circuit described in the Software. Fujitsu does not              
authorize its products for use in any products where a malfunction or       
failure of the Fujitsu product may reasonably be expected to result in      
significant property damage, injury or death ("High Risk Product"). By      
including Fujitsu's product in a High Risk Product, the manufacturer        
of such system or application assumes all risk of such use and in doing     
so agrees to indemnify Fujitsu against all liability.                       

 ******************************************************************************/
/******************************************************************************/
/** \file main.c
 **
 ** A detailed description is available at 
 ** @link mainGroup  description @endlink
 **
 ** History:
 **   - 2017-03-17  V1.0  MSc  Automatic Created
 *****************************************************************************/
#define __MAIN_C__
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/
#include "mcu.h"
#include "base_types.h"
#include "board.h"

/*****************************************************************************/
/* Local pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/


/*****************************************************************************/
/* Global variable definitions (declared in header file with 'extern')       */
/*****************************************************************************/


/*****************************************************************************/
/* Local type definitions ('typedef')                                        */
/*****************************************************************************/


/*****************************************************************************/
/* Local variable definitions ('static')                                     */
/*****************************************************************************/

static volatile uint32_t u32Counter;  //ms counter

/*****************************************************************************/
/* Local function prototypes ('static')                                      */
/*****************************************************************************/


/*****************************************************************************/
/* Function implementation - global ('extern') and local ('static')          */
/*****************************************************************************/

float32_t f32ROld = 0;
float32_t f32GOld = 0;
float32_t f32BOld = 0;
float32_t f32R = 0;
float32_t f32G = 0;
float32_t f32B = 0;
uint8_t u8LedStatus = 0;
uint32_t u32StopLeds = 0;

/**
 *****************************************************************************
 ** 
 **\brief Systick interrupt handler defined by CMSIS
 **
 *****************************************************************************/
void SysTick_Handler(void)
{
	u32Counter++;
        if (u32StopLeds > 0) u32StopLeds--;
        if (((u32Counter % 10) == 0) && (u32StopLeds == 0))
        {
            switch(u8LedStatus)
            {
              case 0:
                f32R += 0.05f;
                if (f32R >= 0.95f)
                {
                    u8LedStatus++;
                }
                break;
              case 1:
                f32R -=  0.05f;
                if (f32R <= 0.05f)
                {
                    u8LedStatus++;
                }
                break; 
              case 2:
                f32G +=  0.05f;
                if (f32G >= 0.95f)
                {
                    u8LedStatus++;
                }
                break;
              case 3:
                f32G -=  0.05f;
                if (f32G <= 0.05f)
                {
                    u8LedStatus++;
                }
                break; 
               case 4:
                f32B +=  0.05f;
                if (f32B >= 0.95f)
                {
                    u8LedStatus++;
                }
                break;
              case 5:
                f32B -=  0.05f;
                if (f32B <= 0.05f)
                {
                    u8LedStatus++;
                }
                break; 
              default:
                u8LedStatus = 0;
                break;
            }
        }
}

void TestAllGPIOs(void)
{
    int i,i2;
    if (*((volatile uint32_t*)0x10001000) != 0x55AA55AA)
    {
        *((volatile uint32_t*)0x10001000) = 0x55AA55AA;
        for(i = 0;i<50;i++)
        {
            if ((i != 20) && (i != 21))
            {
                ApolloGpio_GpioOutputEnable(i,TRUE);
                SET_GPIO(i);
                for(i2 = 0;i2<100000;i2++);
                CLEAR_GPIO(i);
                for(i2 = 0;i2<100000;i2++);
            }
        }
        NVIC_SystemReset();
    }
}

void TestDcDc(void)
{
    ApolloCTimer_PwmSetDutyByPin(LED_RED,1.0f);
    ApolloCTimer_PwmSetDutyByPin(LED_GREEN,1.0f);
    ApolloCTimer_PwmSetDutyByPin(LED_BLUE,0.0f);
    #if defined(APOLLO_H) || defined(APOLLO1_H)
        MCUCTRL->SUPPLYSRC_b.COREBUCKEN = 1;
        MCUCTRL->SUPPLYSRC_b.MEMBUCKEN = 1;
        while(MCUCTRL->SUPPLYSTATUS_b.COREBUCKON == 0) __NOP();
        while(MCUCTRL->SUPPLYSTATUS_b.MEMBUCKON == 0) __NOP();
    #elif defined(APOLLO2_H)
        PWRCTRL->SUPPLYSRC_b.COREBUCKEN = 1;
        PWRCTRL->SUPPLYSRC_b.MEMBUCKEN = 1;
        while((PWRCTRL->POWERSTATUS_b.COREBUCKON != 1) && (PWRCTRL->POWERSTATUS_b.MEMBUCKON != 1)) __NOP();
        PWRCTRL->SUPPLYSRC_b.SWITCH_LDO_IN_SLEEP = 0; // For lowest deep sleep power, make sure we stay in BUCK mode.
    #endif
}

void TestCrystal(void)
{
    ApolloCTimer_PwmSetDutyByPin(LED_RED,1.0f);
    ApolloCTimer_PwmSetDutyByPin(LED_GREEN,0.0f);
    ApolloCTimer_PwmSetDutyByPin(LED_BLUE,1.0f);
    while(CLKGEN->STATUS_b.OSCF == 1) __NOP();
}

/**
 *****************************************************************************
 ** 
 **\brief Main function
 **
 *****************************************************************************/
int main(void)
{
    TestAllGPIOs();
    SystemCoreClockUpdate();                //update clock variable SystemCoreClock (defined by CMSIS)
    SysTick_Config(SystemCoreClock / 1000); //setup 1ms SysTick (defined by CMSIS)
    Board_Init();
    
    TestDcDc();
    TestCrystal();
    
    //application initialization area
    ApolloUart_PutString(BOARD_USBUART_HANDLE,"Hello World\r\n");
    while(1)
    {
        if (ApolloUart_HasChar(BOARD_USBUART_HANDLE))
        {
            ApolloUart_PutChar(BOARD_USBUART_HANDLE,ApolloUart_GetChar(BOARD_USBUART_HANDLE));
            u32StopLeds = 500;
            ApolloCTimer_PwmSetDutyByPin(LED_RED,0.0f);
            ApolloCTimer_PwmSetDutyByPin(LED_GREEN,0.0f);
            ApolloCTimer_PwmSetDutyByPin(LED_BLUE,1.0f);
        }
        if (f32R != f32ROld)
        {
            ApolloCTimer_PwmSetDutyByPin(LED_RED,f32R);
            f32ROld = f32R;
        }
        if (f32G != f32GOld)
        {
            ApolloCTimer_PwmSetDutyByPin(LED_GREEN,f32G);
            f32GOld = f32G;
        }
        if (f32B != f32BOld)
        {
            ApolloCTimer_PwmSetDutyByPin(LED_BLUE,f32B);
            f32BOld = f32B;
        }
        if (ApolloGpio_GpioGet(BOARD_SW1) == 0)
        {
            ApolloCTimer_PwmSetDutyByPin(LED_RED,1.0f);
            ApolloCTimer_PwmSetDutyByPin(LED_GREEN,0.0f);
            ApolloCTimer_PwmSetDutyByPin(LED_BLUE,0.0f);
            u32StopLeds = 500;
        }
        if (ApolloGpio_GpioGet(BOARD_SW2) == 0)
        {
            ApolloCTimer_PwmSetDutyByPin(LED_RED,0.0f);
            ApolloCTimer_PwmSetDutyByPin(LED_GREEN,1.0f);
            ApolloCTimer_PwmSetDutyByPin(LED_BLUE,0.0f);
            u32StopLeds = 500;
        }
        //application code
    }
}



/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

