/******************************************************************************

Copyright (C) 2013-2017, Fujitsu Electronics Europe GmbH or a               
subsidiary of Fujitsu Electronics Europe GmbH.  All rights reserved.        
                                                                            
This software, including source code, documentation and related             
materials ("Software"), is owned by Fujitsu Electronics Europe GmbH or      
one of its subsidiaries ("Fujitsu").
                                                                            
If no EULA applies, Fujitsu hereby grants you a personal, non-exclusive,    
non-transferable license to copy, modify, and compile the                   
Software source code solely for use in connection with Fujitsu's            
integrated circuit products.  Any reproduction, modification, translation,  
compilation, or representation of this Software except as specified         
above is prohibited without the express written permission of Fujitsu.      
                                                                            
Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO                        
WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING,                        
BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED                                
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A                             
PARTICULAR PURPOSE. Fujitsu reserves the right to make                      
changes to the Software without notice. Fujitsu does not assume any         
liability arising out of the application or use of the Software or any      
product or circuit described in the Software. Fujitsu does not              
authorize its products for use in any products where a malfunction or       
failure of the Fujitsu product may reasonably be expected to result in      
significant property damage, injury or death ("High Risk Product"). By      
including Fujitsu's product in a High Risk Product, the manufacturer        
of such system or application assumes all risk of such use and in doing     
so agrees to indemnify Fujitsu against all liability.                       

 ******************************************************************************/
/******************************************************************************/
/** \file SkAmApolloBase.h
 **
 ** A detailed description is available at 
 ** @link SkAmApolloBaseGroup SK-AMAPOLLO-BASE Board Support description @endlink
 **
 ** History:
 **   - 2017-08-28  V1.0  MSc  First Version

 *****************************************************************************/
#ifndef __SKAMAPOLLOBASE_H__
#define __SKAMAPOLLOBASE_H__

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

/**
 ******************************************************************************
 ** \defgroup SkAmApolloBaseGroup SK-AMAPOLLO-BASE Board Support
 **
 ** Provided functions of SkAmApolloBase:
 ** 
 **   
 ******************************************************************************/
//@{

/**
 ******************************************************************************    
 ** \page skamapollobase_module_includes Required includes in main application
 ** \brief Following includes are required
 ** @code   
 ** #include "skamapollobase.h"   
 ** @endcode
 **
 ******************************************************************************/
    
/*****************************************************************************/
/* Include files                                                             */
/*****************************************************************************/

#include "mcu.h"

/*****************************************************************************/
/* Global pre-processor symbols/macros ('#define')                            */
/*****************************************************************************/

#define BOARD_ARDUINO_UNO 1
#define ARDUINO_D0 36
#define ARDUINO_D1 35
#define ARDUINO_D2 25
#define ARDUINO_D3 26
#define ARDUINO_D4 18
#define ARDUINO_D5 19
#define ARDUINO_D6 46
#define ARDUINO_D7 47
#define ARDUINO_D8 48
#define ARDUINO_D9 49
#define ARDUINO_D10 27
#define ARDUINO_D11 10
#define ARDUINO_D12 9
#define ARDUINO_D13 8
#define ARDUINO_D18 6
#define ARDUINO_D19 5

#if defined(APOLLO_H) || defined(APOLLO1_H)
    #define ARDUINO_A0 12
    #define ARDUINO_A1 13
    #define ARDUINO_A2 14
    #define ARDUINO_A3 31
    #define ARDUINO_A4 32
    #define ARDUINO_A5 30 
#elif defined(APOLLO2_H)
    #define ARDUINO_A0 16
    #define ARDUINO_A1 34
    #define ARDUINO_A2 11
    #define ARDUINO_A3 31
    #define ARDUINO_A4 32
    #define ARDUINO_A5 33
#endif
			
#define ARDUINO_RXD ARDUINO_D0
#define ARDUINO_TXD ARDUINO_D1
			
#define ARDUINO_SDA ARDUINO_D18
#define ARDUINO_SCL ARDUINO_D19
    
#define ARDUINO_MISO ARDUINO_D12
#define ARDUINO_MOSI ARDUINO_D11
#define ARDUINO_SCK ARDUINO_D13

#define ARDUINO_SPI_HANDLE IOMSTR1
#define ARDUINO_SPI_CS 4

#define ARDUINO_MISO_FUNC 1
#define ARDUINO_MOSI_FUNC 1
#define ARDUINO_SCK_FUNC 1
#define ARDUINO_SS_FUNC 1
   
#define ARDUINO_I2C_HANDLE IOMSTR0
#define ARDUINO_SDA_FUNC 0
#define ARDUINO_SCL_FUNC 0
    
#define BOARD_BUTTON1 28
#define BOARD_BUTTON2 17

#define BOARD_BTN1 28
#define BOARD_BTN2 17

#define BOARD_SW1 28
#define BOARD_SW2 17

#define BTNSW1 28
#define BTNSW2 17
    
#define BOARD_LED_RED 45
#define BOARD_LED_GREEN 43
#define BOARD_LED_BLUE 42

#define LEDRED 45
#define LEDGREEN 43
#define LEDBLUE 42



#if defined(APOLLO_H) || defined(APOLLO1_H)
    #define BOARD_USBUART_HANDLE UART
    #define BOARD_USBRX_FUNC 2
    #define BOARD_USBTX_FUNC 2
    #define BOARD_USBRX 1
    #define BOARD_USBTX 0
    #define USBRX 1
    #define USBTX 0
#elif defined(APOLLO2_H)
    #define BOARD_USBUART_HANDLE UART0
    #define BOARD_USBRX_FUNC 2
    #define BOARD_USBTX_FUNC 2
    #define BOARD_USBRX 2
    #define BOARD_USBTX 1
    #define USBRX 2
    #define USBTX 1
#endif


/*****************************************************************************/
/* Global type definitions ('typedef')                                        */
/*****************************************************************************/
#define TYPEDEF_PINNAME 1
#define MBED_PINNAME 1
	typedef enum {
			GPIO0 = 0,
			GPIO1 = 1,
			GPIO2 = 2,
			GPIO3 = 3,
			GPIO4 = 4,
			GPIO5 = 5,
			GPIO6 = 6,
			GPIO7 = 7,
			GPIO8 = 8,
			GPIO9 = 9,
			GPIO10 = 10,
			GPIO11 = 11,
			GPIO12 = 12,
			GPIO13 = 13,
			GPIO14 = 14,
			GPIO15 = 15,
			GPIO16 = 16,
			GPIO17 = 17,
			GPIO18 = 18,
			GPIO19 = 19,
			GPIO20 = 20,
			GPIO21 = 21,
			GPIO22 = 22,
			GPIO23 = 23,
			GPIO24 = 24,
			GPIO25 = 25,
			GPIO26 = 26,
			GPIO27 = 27,
			GPIO28 = 28,
			GPIO29 = 29,
			GPIO30 = 30,
			GPIO31 = 31,
			GPIO32 = 32,
			GPIO33 = 33,
			GPIO34 = 34,
			GPIO35 = 35,
			GPIO36 = 36,
			GPIO37 = 37,
			GPIO38 = 38,
			GPIO39 = 39,
			GPIO40 = 40,
			GPIO41 = 41,
			GPIO42 = 42,
			GPIO43 = 43,
			GPIO44 = 44,
			GPIO45 = 45,
			GPIO46 = 46,
			GPIO47 = 47,
			GPIO48 = 48,
			GPIO49 = 49,
			
			D0 = ARDUINO_D0,
			D1 = ARDUINO_D1,
			D2 = ARDUINO_D2,
			D3 = ARDUINO_D3,
			D4 = ARDUINO_D4,
			D5 = ARDUINO_D5,
			D6 = ARDUINO_D6,
			D7 = ARDUINO_D7,
			D8 = ARDUINO_D8,
			D9 = ARDUINO_D9,
			D10 = ARDUINO_D10,
			D11 = ARDUINO_D11,
			D12 = ARDUINO_D12,
			D13 = ARDUINO_D13,
			D14 = ARDUINO_D18,
			D15 = ARDUINO_D19,

			A0 = ARDUINO_A0,
			A1 = ARDUINO_A1,
			A2 = ARDUINO_A2,
			A3 = ARDUINO_A3,
			A4 = ARDUINO_A4,
			A5 = ARDUINO_A5,
			

			USBUART_TX = BOARD_USBRX, 
			USBUART_RX = BOARD_USBTX, 
			
			RXD = D0,
			TXD = D1,
			
			SDA = D14,
			SCL = D15,
                        
                        LED_RED = BOARD_LED_RED,
                        LED_GREEN = BOARD_LED_GREEN,
                        LED_BLUE = BOARD_LED_BLUE, 
                        
			BTN1 = BOARD_BUTTON1, 
                        BTN2 = BOARD_BUTTON2, 
                        
			SW1 = BOARD_BUTTON1, 
                        SW2 = BOARD_BUTTON2, 

			NC = (int)0xFFFFFFFF
	} PinName; 
        

/*****************************************************************************/
/* Global variable declarations ('extern', definition in C source)           */
/*****************************************************************************/



/*****************************************************************************/
/* Global function prototypes ('extern', definition in C source)             */
/*****************************************************************************/

void SkAmaApolloBase_Init(void); 

#ifdef __cplusplus
}
#endif

//@} // SkAmApolloBaseGroup

#endif /*__SKAMAPOLLOBASE_H__*/

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

